package com.bpcl.lpg.core.service.impl;

import org.slf4j.LoggerFactory;
import java.util.Objects;
import org.springframework.http.HttpStatus;
import com.bpcl.facades.payment.refill.RefillBookingData;
import com.bpcl.lpg.facades.data.LpgPaymentRequestData;
import java.util.Date;
import java.util.Calendar;
import java.text.MessageFormat;
import com.bpcl.lpg.core.enums.RefillBookingStatusEnum;
import com.bpcl.lpg.core.model.LpgPaymentInfoModel;
import java.text.ParseException;
import com.bpcl.core.custom.exception.BpclInvalidPaymentResponseException;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.bpcl.lpg.facades.data.LpgPaymentQueryData;
import com.bpcl.lpg.core.pojo.BillDeskResponseMsgData;
import com.bpcl.lpg.core.payment.billdesk.LPGRefundService;
import com.bpcl.lpg.core.service.FetchPaymentStatusService;
import com.bpcl.lpg.core.pojo.LpgPaymentResponseData;
import java.net.http.HttpResponse;
import de.hybris.platform.servicelayer.dto.converter.Converter;
import com.bpcl.lpg.core.service.LpgPayAndBookConnectionService;
import com.bpcl.lpg.core.payment.billdesk.LPGPaymentService;
import com.bpcl.lpg.core.helper.LpgRefillHelper;
import org.slf4j.Logger;
import com.bpcl.lpg.core.service.LpgRefillService;

public class LpgRefillServiceImpl implements LpgRefillService
{
    private static final Logger LOG;
    private LpgRefillHelper lpgRefillHelper;
    private LPGPaymentService lpgPaymentService;
    private LpgPayAndBookConnectionService lpgPayAndBookConnectionService;
    private Converter<HttpResponse<String>, LpgPaymentResponseData> lpgPayBookResponseDataConverter;
    private FetchPaymentStatusService fetchPaymentStatusService;
    private LPGRefundService lpgRefundService;
    
    public boolean confirmRefillPaymentAndUpdate(final BillDeskResponseMsgData responseMsgData, final LpgPaymentQueryData queryData) throws JsonProcessingException, BpclInvalidPaymentResponseException, ParseException {
        final String transactionID = responseMsgData.getCustomerID();
        final HttpResponse<String> httpResponse = this.getRefillData(responseMsgData, queryData);
        final LpgPaymentResponseData refillData = (LpgPaymentResponseData)this.lpgPayBookResponseDataConverter.convert((Object)httpResponse);
        this.updateRefillToPaymentInfo(transactionID, httpResponse, refillData);
        return this.isRefillFailed(httpResponse, refillData);
    }
    
    public void fetchPaymentStatusAndUpdate(final LpgPaymentInfoModel paymentInfo, final Long maxProcessTimeInMs) {
        LpgRefillServiceImpl.LOG.info("Processing the transactionId :: {}", paymentInfo.getTransactionID());
        if (this.isMaxProcessTimeExceeds(paymentInfo, maxProcessTimeInMs)) {
            try {
                this.lpgPaymentService.updateRefillStatus(paymentInfo.getTransactionID(), RefillBookingStatusEnum.MONITOR);
            }
            catch (final BpclInvalidPaymentResponseException exception) {
                LpgRefillServiceImpl.LOG.error("Exception while updating refill status", exception.getCause());
            }
        }
        else {
            this.processInProgressRefill(paymentInfo);
        }
    }
    
    private void processInProgressRefill(final LpgPaymentInfoModel paymentInfo) {
        try {
            final LpgPaymentResponseData responseData = this.fetchPaymentStatusService.fetchPaymentStatus(paymentInfo);
            final String transactionID = paymentInfo.getTransactionID();
            if ("200".equals(responseData.getStatus_Code())) {
                this.updateRefillBookingData(transactionID, responseData);
                LpgRefillServiceImpl.LOG.info("Refill booking is success for the transaction:{} ", transactionID);
            }
            if ("770".equals(responseData.getStatus_Code()) && responseData.getRefund_Status_Code().equals("1")) {
                LpgRefillServiceImpl.LOG.error("Refill booking is failed for the transaction: {} and Initiating the refund.", transactionID);
                this.initiateRefund(paymentInfo);
            }
        }
        catch (final JsonProcessingException | BpclInvalidPaymentResponseException exception) {
            LpgRefillServiceImpl.LOG.error(MessageFormat.format("Exception while processing INPROGRESS Refill {0}", exception.getCause()));
        }
    }
    
    private boolean isMaxProcessTimeExceeds(final LpgPaymentInfoModel paymentInfo, final Long maxProcessTimeInMs) {
        final Date currentDate = Calendar.getInstance().getTime();
        final long difference = currentDate.getTime() - paymentInfo.getTransactionUTimeStamp().getTime();
        return maxProcessTimeInMs < difference;
    }
    
    public void initiateRefund(final LpgPaymentInfoModel paymentInfo) throws JsonProcessingException {
        try {
            this.lpgPaymentService.updateRefillStatus(paymentInfo.getTransactionID(), RefillBookingStatusEnum.FAILED);
            this.lpgRefundService.processRefund(this.lpgRefillHelper.prepareRefundData(paymentInfo), paymentInfo);
        }
        catch (final BpclInvalidPaymentResponseException exception) {
            LpgRefillServiceImpl.LOG.error("InvalidPaymentException", exception.getCause());
        }
    }
    
    private HttpResponse<String> getRefillData(final BillDeskResponseMsgData responseMsgData, final LpgPaymentQueryData queryData) throws JsonProcessingException, ParseException {
        final String transactionID = responseMsgData.getCustomerID();
        final LpgPaymentRequestData lpgPaymentRequestData = this.lpgRefillHelper.prepareLpgPaymentRequestData(responseMsgData, queryData);
        final HttpResponse<String> httpResponse = this.lpgPayAndBookConnectionService.getPayAndBookResponse(lpgPaymentRequestData, "", "confirmRefillPaymentUrl");
        return httpResponse;
    }
    
    private void updateRefillToPaymentInfo(final String transactionID, final HttpResponse<String> httpResponse, final LpgPaymentResponseData refillData) throws BpclInvalidPaymentResponseException {
        if (this.isRefillStatusUnknown(httpResponse, refillData)) {
            this.lpgPaymentService.updateRefillStatus(transactionID, RefillBookingStatusEnum.INPROGRESS);
        }
        else {
            this.updateRefillBookingData(transactionID, refillData);
        }
    }
    
    private void updateRefillBookingData(final String transactionID, final LpgPaymentResponseData refillData) throws BpclInvalidPaymentResponseException {
        final RefillBookingData refillBookingData = this.lpgRefillHelper.prepareRefillBookingData(refillData);
        this.lpgPaymentService.updateRefillStatusByTxnID(transactionID, refillBookingData);
    }
    
    private boolean isRefillStatusUnknown(final HttpResponse<String> httpResponse, final LpgPaymentResponseData refillData) {
        final int OK = HttpStatus.OK.value();
        return Objects.isNull(httpResponse) || httpResponse.statusCode() != OK || (httpResponse.statusCode() == OK && "900".equals(refillData.getStatus_Code()));
    }
    
    private boolean isRefillFailed(final HttpResponse<String> httpResponse, final LpgPaymentResponseData refillData) {
        final boolean refillStatusUnknown = this.isRefillStatusUnknown(httpResponse, refillData);
        return !refillStatusUnknown && !"200".equals(refillData.getStatus_Code()) && !"201".equals(refillData.getStatus_Code());
    }
    
    public void setLpgRefillHelper(final LpgRefillHelper lpgRefillHelper) {
        this.lpgRefillHelper = lpgRefillHelper;
    }
    
    public void setLpgPaymentService(final LPGPaymentService lpgPaymentService) {
        this.lpgPaymentService = lpgPaymentService;
    }
    
    public void setLpgPayAndBookConnectionService(final LpgPayAndBookConnectionService lpgPayAndBookConnectionService) {
        this.lpgPayAndBookConnectionService = lpgPayAndBookConnectionService;
    }
    
    public void setLpgPayBookResponseDataConverter(final Converter<HttpResponse<String>, LpgPaymentResponseData> lpgPayBookResponseDataConverter) {
        this.lpgPayBookResponseDataConverter = lpgPayBookResponseDataConverter;
    }
    
    public void setFetchPaymentStatusService(final FetchPaymentStatusService fetchPaymentStatusService) {
        this.fetchPaymentStatusService = fetchPaymentStatusService;
    }
    
    public void setLpgRefundService(final LPGRefundService lpgRefundService) {
        this.lpgRefundService = lpgRefundService;
    }
    
    static {
        LOG = LoggerFactory.getLogger(LpgRefillServiceImpl.class);
    }
}
